@@ -1,7 +1,7 @@
 /*************************************************************************
  *
  * Copyright (c) 2010 Kohei Yoshida
- * 
+ *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
  * files (the "Software"), to deal in the Software without
@@ -10,10 +10,10 @@
  * copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following
  * conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be
  * included in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -42,7 +42,7 @@ typedef void (test_func_type)(matrix_density_t);
 
 /**
  * Run specified test on all matrix density types.
- *  
+ *
  * @param func function pointer to the test function to be performed.
  */
 void run_tests_on_all_density_types(test_func_type* func)
@@ -182,7 +182,7 @@ bool verify_assign(const _Mx& mx1, const _Mx& mx2)
             matrix_element_t elem_type = mx1.get_type(i, j);
             if (elem_type != mx2.get_type(i, j))
                 return false;
-            
+
             switch (elem_type)
             {
                 case element_boolean:
@@ -217,37 +217,48 @@ bool verify_assignment(const _Mx& mx1, const _Mx& mx2)
 
 void mtm_test_resize(matrix_density_t density)
 {
-    StackPrinter __stack_printer__("::mtm_test_resize");
+    stack_printer __stack_printer__("::mtm_test_resize");
     print_mx_density_type(density);
     pair<size_t,size_t> mxsize;
+    pair<size_t,size_t> mxstoragesize;
 
     {
         // Start with an empty matrix, and resize into a non-empty one.
         mx_type mx(0, 0, density);
         mxsize = mx.size();
+        mxstoragesize = mx.get_storage_size();
         assert(mxsize.first == 0);
         assert(mxsize.second == 0);
+        assert(mxsize == mxstoragesize);
         mx.resize(1, 1);
         mxsize = mx.size();
+        mxstoragesize = mx.get_storage_size();
         assert(mxsize.first == 1);
         assert(mxsize.second == 1);
+        assert(mxsize == mxstoragesize);
         // Back to an empty matrix again.
         mx.resize(0, 0);
         mxsize = mx.size();
+        mxstoragesize = mx.get_storage_size();
         assert(mxsize.first == 0);
         assert(mxsize.second == 0);
+        assert(mxsize == mxstoragesize);
         // Resize to a non-square matrix.
         mx.resize(5, 10);
         mxsize = mx.size();
+        mxstoragesize = mx.get_storage_size();
         assert(mxsize.first == 5);
         assert(mxsize.second == 10);
+        assert(mxsize == mxstoragesize);
     }
 
     mx_type mx(3, 3, density);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 3);
     assert(mxsize.second == 3);
+    assert(mxsize == mxstoragesize);
 
     mx.set_string(0, 0, new string("test"));
     mx.set_numeric(0, 1, 2.3);
@@ -260,14 +271,20 @@ void mtm_test_resize(matrix_density_t density)
     mx.resize(6, 4);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 6);
     assert(mxsize.second == 4);
+    assert(mxsize == mxstoragesize);
+
     mx.resize(6, 6);
     mx.set_boolean(5, 5, true);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 6);
     assert(mxsize.second == 6);
+    assert(mxsize == mxstoragesize);
+
     mx.resize(3, 6);
     mx.set_numeric(2, 2, 4.2);
     mx.set_numeric(2, 3, 3.2);
@@ -275,26 +292,32 @@ void mtm_test_resize(matrix_density_t density)
     mx.set_numeric(2, 5, 1.2);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 3);
     assert(mxsize.second == 6);
+    assert(mxsize == mxstoragesize);
 
     mx.resize(3, 3);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 3);
     assert(mxsize.second == 3);
+    assert(mxsize == mxstoragesize);
 
     mx.resize(0, 0);
     mx.dump();
     mxsize = mx.size();
+    mxstoragesize = mx.get_storage_size();
     assert(mxsize.first == 0);
     assert(mxsize.second == 0);
+    assert(mxsize == mxstoragesize);
     assert(mx.empty());
 }
 
 void mtm_test_value_store(matrix_density_t density)
 {
-    StackPrinter __stack_printer__("::mtm_test_value_store");
+    stack_printer __stack_printer__("::mtm_test_value_store");
     print_mx_density_type(density);
     mx_type mx(5, 5, density);
     mx.dump();
@@ -303,7 +326,7 @@ void mtm_test_value_store(matrix_density_t density)
     assert(mxsize.second == 5);
     assert(!mx.empty());
 
-    // Make sure all elements have been initialized properly according to the 
+    // Make sure all elements have been initialized properly according to the
     // matrix type.
     for (size_t i = 0; i < 5; ++i)
     {
@@ -315,7 +338,7 @@ void mtm_test_value_store(matrix_density_t density)
                 case matrix_density_filled_zero:
                 case matrix_density_sparse_zero:
                 {
-                    // filled matrices are initialized to numeric elements 
+                    // filled matrices are initialized to numeric elements
                     // having a value of 0.
                     assert(elem_type == element_numeric);
                     double val = mx.get_numeric(i, j);
@@ -410,7 +433,7 @@ void print_transposed_mx(const _Mx& original, const _Mx& transposed)
 
 void mtm_test_transpose(matrix_density_t density)
 {
-    StackPrinter __stack_printer__("::mtm_test_transpose");
+    stack_printer __stack_printer__("::mtm_test_transpose");
     print_mx_density_type(density);
 
     {
@@ -457,7 +480,7 @@ void mtm_test_transpose(matrix_density_t density)
 
 void mtm_test_initial_elements()
 {
-    StackPrinter __stack_printer__("::mtm_test_initial_elements");
+    stack_printer __stack_printer__("::mtm_test_initial_elements");
     {
         mx_type mx(3, 3, matrix_density_filled_zero);
         mx.dump();
@@ -497,7 +520,7 @@ void mtm_test_initial_elements()
 
 void mtm_test_numeric_matrix()
 {
-    StackPrinter __stack_printer__("::mtm_test_numeric_matrix");
+    stack_printer __stack_printer__("::mtm_test_numeric_matrix");
     {
         print_mx_density_type(matrix_density_filled_zero);
         mx_type mx(3, 3, matrix_density_filled_zero);
@@ -553,7 +576,7 @@ void mtm_test_numeric_matrix()
 
 void mtm_test_assign(matrix_density_t dens1, matrix_density_t dens2)
 {
-    StackPrinter __stack_printer__("::mtm_test_assign");
+    stack_printer __stack_printer__("::mtm_test_assign");
     print_mx_density_type(dens1);
     print_mx_density_type(dens2);
 
@@ -604,7 +627,7 @@ void mtm_test_assign(matrix_density_t dens1, matrix_density_t dens2)
 
 void mtm_test_assignment(matrix_density_t density)
 {
-    StackPrinter __stack_printer__("::mtm_test_assignment");
+    stack_printer __stack_printer__("::mtm_test_assignment");
     print_mx_density_type(density);
     mx_type mx1(0, 0, density), mx2(3, 3, density);
     mx2.set(0, 0, 3.4);
@@ -642,7 +665,7 @@ void mtm_test_assignment(matrix_density_t density)
 
 void mtm_test_flag_storage(matrix_density_t density)
 {
-    StackPrinter __stack_printer__("::mtm_test_flag_storage");
+    stack_printer __stack_printer__("::mtm_test_flag_storage");
     print_mx_density_type(density);
     mx_type mx(3, 3, density);
     mx.dump();
@@ -698,7 +721,7 @@ void traverse_itr_access(typename _StoreType::const_itr_access& itr_access)
 template<typename _FilledStoreType>
 void mtm_test_iterator_access_filled(size_t rows, size_t cols)
 {
-    StackPrinter __stack_printer__("::mtm_test_iterator_access_filled");
+    stack_printer __stack_printer__("::mtm_test_iterator_access_filled");
     typedef _FilledStoreType store_type;
 
     store_type store(rows, cols, matrix_init_element_zero);
@@ -712,8 +735,8 @@ void mtm_test_iterator_access_filled(size_t rows, size_t cols)
 
 void mtm_test_iterator_access_sparse()
 {
-    StackPrinter __stack_printer__("::mem_test_iterator_access_sparse");
-    typedef storage_sparse<mx_type> store_type;
+    stack_printer __stack_printer__("::mem_test_iterator_access_sparse");
+    typedef mx_type::sparse_storage_type store_type;
     store_type store(5, 5, matrix_init_element_empty);
     {
         store_type::const_itr_access* itr_access = store.get_const_itr_access();
@@ -763,7 +786,7 @@ private:
 
 void mtm_test_const_iterator()
 {
-    StackPrinter __stack_printer__("::mtm_test_const_iterator");
+    stack_printer __stack_printer__("::mtm_test_const_iterator");
     // Test with an empty matrix.
     mx_type mx(0, 0, matrix_density_filled_zero);
     assert(mx.empty());
@@ -848,33 +871,34 @@ void mtm_test_const_iterator()
 /**
  * Measure the performance of object instantiation for filled storage.
  */
-void mtm_perf_test_filled_storage_creation()
+void mtm_perf_test_storage_creation()
 {
-    StackPrinter __stack_printer__("::mtm_perf_test_filled_storage_creation");
     cout << "measuring performance on matrix object creation." << endl;
-    size_t rowsize = 1000;
+    size_t rowsize = 5000;
     size_t obj_count = 30000;
     cout << "row size: " << rowsize << "  object count: " << obj_count << endl;
-    for (size_t colsize = 1; colsize <= 3; ++colsize)
+    cout << "--- filled zero" << endl;
+    for (size_t colsize = 1; colsize <= 5; ++colsize)
     {
-        StackPrinter __stack_printer2__("::mtm_perf_test_filled_storage_creation::group");
-        cout << "column size: " << colsize << endl;
+        stack_watch sw;
         for (size_t i = 0; i < obj_count; ++i)
             mx_type mx(rowsize, colsize, matrix_density_filled_zero);
+
+        cout << "column size: " << colsize << "  duration: " << sw.get_duration() << " sec" << endl;
     }
+    cout << endl;
 }
 
-void mtm_perf_test_filled_storage_set_numeric()
+void mtm_perf_test_storage_set_numeric()
 {
-    StackPrinter __stack_printer__("::mtm_perf_test_filled_storage_set_numeric");
     cout << "measuring performance on matrix object creation and populating it with numeric data." << endl;
-    size_t rowsize = 1000;
+    size_t rowsize = 3000;
     size_t obj_count = 30000;
     cout << "row size: " << rowsize << "  object count: " << obj_count << endl;
-    for (size_t colsize = 1; colsize <= 3; ++colsize)
+    cout << "--- filled zero" << endl;
+    for (size_t colsize = 1; colsize <= 5; ++colsize)
     {
-        StackPrinter __stack_printer2__("::mtm_perf_test_filled_storage_set_numeric::group");
-        cout << "column size: " << colsize << endl;
+        stack_watch sw;
         for (size_t i = 0; i < obj_count; ++i)
         {
             mx_type mx(rowsize, colsize, matrix_density_filled_zero);
@@ -884,7 +908,9 @@ void mtm_perf_test_filled_storage_set_numeric()
                     mx.set_numeric(row, col, 1.0);
             }
         }
+        cout << "column size: " << colsize << "  duration: " << sw.get_duration() << " sec" << endl;
     }
+    cout << endl;
 }
 
 int main(int argc, char** argv)
@@ -899,43 +925,37 @@ int main(int argc, char** argv)
         run_tests_on_all_density_types(mtm_test_value_store);
         run_tests_on_all_density_types(mtm_test_transpose);
         run_tests_on_all_density_types(mtm_test_assignment);
-    
+
         mtm_test_initial_elements();
         mtm_test_numeric_matrix();
         mtm_test_assign(matrix_density_filled_zero, matrix_density_filled_zero);
         mtm_test_assign(matrix_density_filled_empty, matrix_density_filled_zero);
         mtm_test_assign(matrix_density_filled_zero, matrix_density_filled_empty);
         mtm_test_assign(matrix_density_filled_empty, matrix_density_filled_empty);
-    
+
         run_tests_on_all_density_types(mtm_test_flag_storage);
-    
-        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(1, 1);
-        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(3, 1);
-        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(1, 3);
-        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(3, 3);
-        mtm_test_iterator_access_filled<storage_filled_nested_array<mx_type> >(0, 0);
 
-        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(1, 1);
-        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(3, 1);
-        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(1, 3);
-        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(3, 3);
-        mtm_test_iterator_access_filled<storage_filled_linear<mx_type> >(0, 0);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_type>(1, 1);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_type>(3, 1);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_type>(1, 3);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_type>(3, 3);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_type>(0, 0);
+
+        mtm_test_iterator_access_filled<mx_type::filled_storage_zero_type>(1, 1);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_zero_type>(3, 1);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_zero_type>(1, 3);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_zero_type>(3, 3);
+        mtm_test_iterator_access_filled<mx_type::filled_storage_zero_type>(0, 0);
 
-        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(1, 1);
-        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(3, 1);
-        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(1, 3);
-        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(3, 3);
-        mtm_test_iterator_access_filled<storage_filled_linear_zero<mx_type> >(0, 0);
-    
         mtm_test_iterator_access_sparse();
-    
+
         mtm_test_const_iterator();
     }
 
     if (opt.test_perf)
     {
-        mtm_perf_test_filled_storage_creation();
-        mtm_perf_test_filled_storage_set_numeric();
+        mtm_perf_test_storage_creation();
+        mtm_perf_test_storage_set_numeric();
     }
 
     cout << "Test finished successfully!" << endl;
